module GenomeLib
  class VirtGuest
    attr_reader :name, :uuid, :hostname, :ipaddress, :appliance, :status, :arch

    def initialize(name, uuid, hostname, ipaddress, appliance, status, arch)
      @name = name
      @uuid = uuid
      @hostname = hostname
      @ipaddress = ipaddress
      @appliance = appliance
      @status = status
      @arch = arch
    end

    def running?
      @status == "on"
    end
  end

  class Nodeinfo
    attr_reader :cpu_model, :memsize

    def initialize(cpu_model, memsize)
      @cpu_model = cpu_model
      @memsize = memsize.to_i
    end
  end

  class Dominfo
    attr_reader :used_mem

    def initialize(used_mem)
      @used_mem = used_mem.to_i
    end
  end

  class Virsh
    VIRSH = "virsh"

    def list(options={})
      defaults = {:all => false}
      opts = defaults.merge(options)
      
      args = []
      args << "--all" if opts[:all] 
      parse_list(run_command('list', args))
    end

    def nodeinfo
      out = run_command('nodeinfo', [])
      Nodeinfo.new(parse_cpu_model(out), parse_mem_size(out))
    end

    def dominfo(dom)
      out = run_command('dominfo', [dom])
      Dominfo.new(parse_used_mem(out))
    end

    def start(uuid)
      name = get_name(uuid)
      args = []
      args << name
      run_command('start', args)
      return $?.exitstatus
    end

    def shutdown(uuid)
      name = get_name(uuid)
      args = []
      args << name
      run_command('shutdown', args)
      return $?.exitstatus
    end

    private 

    def run_command(cmd, *args)
      `#{VIRSH} #{cmd} #{args.join(" ")}`.split("\n")
    end

    def parse_list(output)
      # Chop off the first two lines
      guests = output[2..-1].map do |line|
        line =~ /\s*(?:\d+|[-])\s([-0-9A-Za-z_\.]+)\s+(.*)$/
        name = $1
        next if name == "Domain-0" # This causes 'nil' to be returned

        # TODO: It would be better to use a boolean here
        status = case $2 
          when /running|blocked/ then "on" 
          when /no state/ then "unknown"
          else "off"
        end

        VirtGuest.new(name, get_uuid(name), nil, nil, nil, status, nil)
      end.reject {|g| g.nil?}

      return guests
    end

    def parse_cpu_model(output)
      parse_info(output, "CPU model", /\w+/)
    end

    def parse_mem_size(output)
      parse_info(output, "Memory size", /\d+/)
    end

    def parse_used_mem(output)
      parse_info(output, "Used memory", /\d+/)
    end

    def parse_info(output, key, regex)
      output.find {|l| l =~ /^#{key}:\s+(#{regex}).*$/}
      return $1
    end
   
    def get_uuid(name)
      `virsh domuuid #{name}`.strip
    end

    # Get the name of a domain based
    # on the dom id or the dom uuid
    # See "virsh help domname"
    def get_name(id)
      `virsh domname #{id}`.strip
    end
  end
end
